import 'dart:async';

import 'package:flutter/services.dart';

enum AuthorizationType {
  always,
  inUse
}

enum AuthorizationStatus {
  authorizedAlways,
  authorizedWhenInUse,
  denied,
  notDetermined,
  restricted
}

enum AccuracyType {
  bestForNavigation,
  best,
  kilometer,
  hundredMeters,
  threeKilometers,
  nearestTenMeters
}

class AJLocation {
  static const MethodChannel _channel = const MethodChannel('ajlocation');
  static const EventChannel _eventChannelLocationChange = const EventChannel("ajlocation.locationChange");

  static StreamController<Location> _onLocationChangeController = StreamController<Location>();

  ///Stream that will return new location everytime it changes. You can listen to this stream to get the latest [Location], also works in background mode.
  static Stream<Location> onLocationChange = _onLocationChangeController.stream;

  static Stream _onLocationChange = _eventChannelLocationChange.receiveBroadcastStream();
  static StreamSubscription _onLocationChangeSubscription;

  ///Request authorization for the location request. Will return [AuthorizationStatus] if the location fetching is [allowed] or [declined].
  static Future<AuthorizationStatus> requestAuthorization({AuthorizationType type = AuthorizationType.always}) async {
    final int response = await _channel.invokeMethod("requestAuthorization", type.index);
    return AuthorizationStatus.values[response];
  }

  ///Get current authorization status.
  static Future<AuthorizationStatus> get currentAuthorizationStatus async {
    final int response = await _channel.invokeMethod("authorizationStatus");
    return AuthorizationStatus.values[response];
  }

  static setConfiguartion({AccuracyType accuracy = AccuracyType.bestForNavigation, double distanceFilter}) {
    _channel.invokeMethod("setConfiguration", {
      "accuracy": accuracy.index,
      "distanceFilter": distanceFilter
    });
  }

  ///Start updating location.
  static void startUpdatingLocation() {
    _channel.invokeMethod("startUpdatingLocation");
    _listenToLocationUpdates();
  }
  
  ///Stop updating location.
  static void stopUpdatingLocation() {
    _channel.invokeMethod("stopUpdatingLocation");
    _onLocationChangeSubscription.cancel();
  }

  static _listenToLocationUpdates() {
    _onLocationChangeSubscription = _onLocationChange.listen((data) {
      final newLocation = Location.fromMap(data);
      _onLocationChangeController.add(newLocation);
    });
  }
}

// Location Wrapper

class Location {
  double altitude;
  double latitude;
  double longitude;
  double speed;
  DateTime timestamp;
  double accuracy;

  Location({double altitude, double latitude, double longitude, double speed, DateTime timeStamp, double accuracy}) {
    this.altitude = altitude;
    this.latitude = latitude;
    this.longitude = longitude;
    this.speed = speed;
    this.timestamp = timestamp;
    this.accuracy = accuracy;
  }

  Location.fromMap(Map<dynamic, dynamic> map) {
    this.altitude = map["altitude"];
    this.latitude = map["latitude"];
    this.longitude = map["longitude"];
    this.speed = map["speed"];
    this.timestamp = DateTime.fromMillisecondsSinceEpoch(((map["timestamp"] as double) * 1000).toInt());
    this.accuracy = map["accuracy"];
  }
}

